set(:db_defaults) do
  {
    'username' => db_username,
    'password' => db_password,
    'adapter' => 'postgresql',
    'encoding' => 'UTF8',
    'host' => db_server,
    'database' => db_name,
    'port' => db_port
  }
end

namespace :deploy do
  def with_tempfile_on_remote_server(&block)
    tempfile_name = capture('mktemp -t preparedb.XXXXXXXX').strip

    yield tempfile_name
  ensure
    run "rm #{tempfile_name}"
  end

  def db_connection
    fetch(:db_connection, {})
  end

  def dbinfo
    @dbinfo ||= db_defaults.merge(db_connection)
    unless stage == 'production'
      @dbinfo['database'] = '%s-%s' % [db_name, stage]
    end
    @dbinfo
  end

  def run_with_password(cmd, pwd)
    once = false
    run cmd do |ch, stream, data|
      $stdout.write(data)
      unless once
        ch.send_data pwd + "\n"
        $stdout.write('[not shown]')
        once = true
      end
    end
  end

  namespace :db do
    desc "Regenerate deployment database.yml"
    task :copy_database_yml do
      put({ rails_env.to_s => dbinfo }.to_yaml, "#{release_path}/config/database.yml")
    end

    desc "Drop and recreate stage DB"
    task :create do
      if reference_db_stages.include? stage.to_s
        puts "Cloning reference database!"
        raise "Your database name #{dbinfo['database']} looks wrong" unless dbinfo['database'] =~ /^[\w-]+-#{stage}$/

        with_tempfile_on_remote_server do |tf|
          referencedb = "%s-reference" % db_name

          popt = [['host', 'h'], ['username', 'U'], ['port', 'p']].
            select { |key, flag| dbinfo[key] }.
            map { |key, flag| '-%s "%s"' % [flag, dbinfo[key]] } * ' ' +
            ' -W'

          sql = <<-SQL % [referencedb, dbinfo['database'], dbinfo['database']]
            select killusers('%s','%s');
            drop database "%s";
            SQL
          put sql, tf

          run_with_password %{psql postgres #{popt} -a -f #{tf} 2>&1}, dbinfo['password']
          run_with_password %{createdb "#{dbinfo['database']}" #{popt} -T "#{referencedb}" 2>&1}, dbinfo['password']
        end 
      else
        puts "Not cloning reference database, because the #{stage} stage is not in reference_db_stages (#{reference_db_stages})."
      end
    end 

    desc "Import seed data"
    task :seed do
      if reference_db_stages.include? stage.to_s
        puts "Seeding database from db/seeds.rb"
        run "cd #{release_path} && env RAILS_ENV=#{rails_env} bundle exec rake db:seed"
      else
        puts "Database is only seeded from db/seeds.rb for stages in reference_db_stages."
      end
    end
  end
end
